<?php
class ExtAuthorisationModNinja extends CoreAuthorisationModule {
	public $rolesConfigurable = false;
	private $op5auth = null;
	private $config = null;

	public function __construct() {
		$this->op5auth     = Op5Auth::instance();
		$this->op5conf     = Op5Config::instance();
		$this->config      = $this->op5conf->getConfig('nagvis'); // Using multiple auth instances is bad for your health, but this is just for reading...
		$this->custom_conf = $this->op5conf->getConfig('nagvis_custom');
	}
	/**
	 * Creates a permission table with full permissions.
	 *
	 * @param number $level Levels left to fill
	 * @return array Nagvis permission table that gives full permission of a
	 *               subtree
	 */
	private function fullPerm( $level=3 ) {
		if( $level <= 0 ) return array('*'); /* Just dont make it empty, array_filter should not remove this */
		return array( '*' => $this->fullPerm($level-1) );
	}

	/**
	 * Prepare the permissions array, taken form the nagvis_custom yml file and
	 * convert it so it can be interpretable from nagvis.
	 *
	 * This means, expand allow_all tag to array( '*' => ... ) with a depth of 3
	 * and remove empty permission fields
	 *
	 * @param array $permtbl table from yml
	 * @param number $level Levels left to recurse, used internally
	 * @return array output table
	 */
	private function preparePerm( $permtbl, $level=3 ) {
		$outp = array();
		if( isset( $permtbl['allow_all'] ) && $permtbl['allow_all'] ) {
			return $this->fullPerm( $level );
		}
		foreach( $permtbl as $perm => $subobj ) {
			if( is_array( $subobj ) ) {
				$outp[$perm] = $this->preparePerm( $subobj, $level-1 );
			}
		}
		/* If no permissions given under the element, remove it not to confuse nagvis */
		return array_filter( $outp );
	}

	/**
	 * Parses permissions from configuration and current auth status.
	 *
	 * @return array Nagvis permission table
	 */
	public function parsePermissions($sUsername = null) {

		if( $this->op5auth->authorized_for( 'nagvis_admin' ) ) {
			/* don't auth to logout, change password, manage users/roles */
			return array(
				'AutoMap'=>array('*'=>array('*'=>array())),
				'General'=>array('*'=>array('*'=>array())),
				'ManageBackgrounds'=>array('*'=>array('*'=>array())),
				'ManageShapes'=>array('*'=>array('*'=>array())),
				'MainCfg'=>array('*'=>array('*'=>array())),
				'Search'=>array('*'=>array('*'=>array())),
				'Map'=>array('*'=>array('*'=>array())),
				'Rotation'=>array('*'=>array('*'=>array())),
				'Overview'=>array('*'=>array('*'=>array())),
				'User'=>array('*'=>array('*'=>array()))
			);
		}

		$map_perm = array();

		$groups_per_type = array(
			'auth_groups'    => $this->op5auth->get_user()->get_groups(),
			'contact_groups' => $this->op5auth->get_user()->get_contact_groups()
		);

		/* Iterate through auth groups and contact groups (grouptypes) */
		foreach( $groups_per_type as $grouptype => $groups ) {
			if( isset( $this->config[$grouptype])) {
				/* Iterate trhough the groups */
				foreach( $groups as $group ) {
					if( isset( $this->config[$grouptype][$group] ) ) {
						/* Iterate through the maps related to the groups */
						foreach( $this->config[$grouptype][$group] as $map => $perms ) {
							/* Iterate through the permissions given to the map */
							foreach( $perms as $perm ) {
								$map_perm[$perm][$map] = array();
							}
						}
					}
				}
			}
		}

		if ($this->op5auth->authorized_for( 'nagvis_view' ) ) {
			$map_perm['view']   = array('*'=>array());
		}
		if ($this->op5auth->authorized_for( 'nagvis_edit' ) ) {
			$map_perm['edit']   = array('*'=>array());
		}
		if ($this->op5auth->authorized_for( 'nagvis_add_delete' ) ) {
			$map_perm           = array('*'=>array('*'=>array()));
		}

		$perm = array(
			'Overview' => array( 'view' => array( '*' => array() ) ),
			'General'  => array( '*'    => array( '*' => array() ) ),
			'AutoMap'  => $map_perm,
			'Map'      => $map_perm,
			'Rotation' => $map_perm,
			'User'=>array('*'=>array('*'=>array())),
		);

		/* Fetch custom configuration overlay */
		$overlay = array();

		/* Fill with custom permissions */
		if( !empty( $this->custom_conf ) ) {
			/* Iterate through auth groups and contact groups (grouptypes) */
			foreach( $groups_per_type as $grouptype => $groups ) {
				if( isset( $this->custom_conf[$grouptype])) {
					foreach( $groups as $group ) {
						if( isset( $this->custom_conf[$grouptype][$group] ) ) {
							$overlay = array_merge_recursive( $overlay,
									$this->custom_conf[$grouptype][$group] );
						}
					}
				}
			}
		}
		$overlay = $this->preparePerm( $overlay );

		/* Merge custom configuration overlay and gui configuration */
		return array_merge_recursive( $perm, $overlay );
	}

    public function getUserId($username) {
        return $username;
    }

    public function renameMapPermissions($old_name, $new_name) {
        return false;
    }

    public function deletePermission($mod, $name) {
        return false;
    }

    public function createPermission($mod, $name) {
        return false;
    }

    public function deleteRole($roleId) {
        return false;
    }

    public function deleteUser($userId) {
        return false;
    }

    public function updateUserRoles($userId, $roles) {
        return false;
    }

    public function updateRolePerms($roleId, $perms) {
        return false;
    }

    public function createRole($name) {
        return false;
    }

    public function getUserRoles($userId) {
        return Array();
    }

    public function getAllRoles() {
        return Array();
    }

    public function getRoleId($sRole) {
        return false;
    }

    public function getAllPerms() {
        return array();
    }

    public function getRolePerms($roleId) {
        return array();
    }

    public function checkRoleExists($name) {
        return false;
    }
}

